/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

package org.greenstone.gatherer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.*;
import javax.swing.JApplet;
import java.applet.*;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.feedback.ActionRecorderDialog;
import org.greenstone.gatherer.gui.GUIManager;
import org.greenstone.gatherer.gui.Splash;
import org.greenstone.gatherer.util.Utility;

public class GathererApplet extends JApplet implements ActionListener
{
    Gatherer gatherer;
    GUIManager g_man;
    Dimension size = new Dimension(800, 540);

    protected String fullLibraryURL(String address) 
    {
	String full_address = "";

        // make sure the URL has protocol, host, and file
        if (address.startsWith("http:")) {
            // the address has all the necessary components
	    full_address = address;
        } 
	else if (address.startsWith("/")) {
            // there is not protocol and host
            URL document = getDocumentBase();
	    int port_no = document.getPort();

            String port = (port_no>0) ? ":" + port_no : "";
            full_address = "http://" + document.getHost() + port  + address;
	}

	return full_address;
    }

    public void init()
    {
	try {
	    // Work-around to reduce number of 
	    // 'Could not lock user prefs' error messages.
	    // Thanks to Walter Schatz from the java forums.
	    System.setProperty("java.util.prefs.syncInterval","2000000");
	}

	catch (Exception exception) {
	    // convenient way of finding out if user has agreed to
	    // to requested security settings

	    JLabel lab 
		= new JLabel("Greenstone Librarian Interface Applet deactivated",
			     JLabel.CENTER);
	    getContentPane().add(lab);
	    return;
	}

	// Ensure platform specific LAF
	try {
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	}
	catch(Exception exception) {
	    exception.printStackTrace();
	}

	String gwcgi = getParameter("gwcgi");
	String library_cgi = fullLibraryURL(gwcgi);
	int cgi_base_cut = library_cgi.lastIndexOf('/');
	String cgi_base = library_cgi.substring(0,cgi_base_cut+1);

	gatherer = new Gatherer();

	String gli_user_folder = Utility.getGLIUserFolder().toString();

	String[] args = { "-gsdl", gli_user_folder,
			  "-library", library_cgi,
	                };

	GetOpt go = new GetOpt(args);

	if (go.feedback_enabled) {
	    // If feedback is enabled, set up the recorder dialog.  
	    // Use the default locale for now - this will be changed by the
	    // Gatherer run method
	    Locale currLocale = Locale.getDefault(); 
	    ActionRecorderDialog dlg = new ActionRecorderDialog (currLocale);
	    gatherer.feedback_enabled = true;
	    gatherer.feedback_dialog = dlg;
	}
	
	Gatherer.isGsdlRemote = true;
	Gatherer.cgiBase = cgi_base;

	Utility.BASE_DIR = go.gsdl_path;
	if (!Utility.BASE_DIR.endsWith(File.separator)) {
	    Utility.BASE_DIR += File.separator;
	}

	Utility.CACHE_DIR    = Utility.BASE_DIR + "cache"    + File.separator;
	Utility.HELP_DIR     = Utility.BASE_DIR + "help"     + File.separator;
	Utility.METADATA_DIR = Utility.BASE_DIR + "metadata" + File.separator;
	Utility.RECYCLE      = Utility.BASE_DIR + "recycle"  + File.separator;
	Utility.RES_DIR      = Utility.BASE_DIR + "resource" + File.separator;
	Utility.WELCOME_DIR  = Utility.BASE_DIR + "welcome"  + File.separator;

	Configuration.TEMPLATE_CONFIG_XML = "xml/configRemote.xml";
	Configuration.CONFIG_XML = "configRemote.xml";
	Configuration.GS3_CONFIG_XML = "config3Remote.xml";

	File gli_user_dir = new File(gli_user_folder);
	if (!gli_user_dir.exists()) {
	    gli_user_dir.mkdirs();
	}

	File col_dir = new File(Utility.getCollectDir(go.gsdl_path));
	if (!col_dir.exists()) {
	    col_dir.mkdir();
	}

	File metadata_directory = new File(Utility.METADATA_DIR);
	if (!metadata_directory.exists()) {
	    // digout metadata.zip from JAR file and unzip it
	    Utility.unzipFromJar(Utility.METADATA_ZIP,gli_user_folder);
	}

	
	File plug_dat = new File(Utility.BASE_DIR + "plugins.dat");
	if (!plug_dat.exists()) {
	    Utility.extractFromJar("plugins.dat",Utility.BASE_DIR,false);
	}

	File class_dat = new File(Utility.BASE_DIR + "classifiers.dat");
	if (!class_dat.exists()) {
	    Utility.extractFromJar("classifiers.dat",Utility.BASE_DIR,false);
	}
	

	g_man = gatherer.init(size, go.gsdl_path, go.gsdl3_path, 
				    go.exec_path, go.debug, go.perl_path, 
				    go.no_load, go.filename, go.site_name, 
				    go.servlet_path, go.mirroring_enabled, 
				    go.wget_version_str, go.wget_path);


	JButton bttn = new JButton("Launch Greenstone Librarian Interface ...");
	bttn.addActionListener(this);
	getContentPane().add(bttn);
    }



    public void start() 
    {
	System.err.println("Start called");
    }



    public void stop()
    {
	System.err.println("Stop called");

    }

    public void destroy()
    {
	System.err.println("Destroy called");
	gatherer.exit();
	System.err.println("Done gatherer exit.");
    }

    public void actionPerformed(ActionEvent e)
    {
	Splash splash = null;

	gatherer.run(size, splash, g_man);
    }
}
